import datetime
from pygeodesy.ellipsoidalVincenty import LatLon

from .utils import price_check, price_check_dict


class StatumRunner:
    def __init__(self, race_change, min_speed, max_speed):
        self.race_change = race_change
        self.min_speed = min_speed
        self.max_speed = max_speed
        self.runner_book = None
        self._active = True
        self._removed = False
        self._price_removed = False
        self.position = LatLon(race_change.lat, race_change.long)

    def __call__(self, race_change, runner_book):
        self._process_runner_book(runner_book)
        self.race_change = race_change
        self.position = LatLon(race_change.lat, race_change.long)

    def check_active(self):
        """ Check that speed looks valid.
        """
        if (
            self.race_change
            and self.min_speed < self.race_change.speed < self.max_speed
        ):
            self._active = True
        else:
            self._active = False

    def _process_runner_book(self, runner_book):
        self.runner_book = runner_book
        if isinstance(runner_book, dict):
            status = runner_book["status"]
            back = price_check_dict(runner_book["ex"]["availableToBack"], 0)
        else:
            status = runner_book.status
            back = price_check(runner_book.ex.available_to_back, 0)

        if status != "ACTIVE":
            self._removed = True
        else:
            self._removed = False  # just in case there is an accidental removal
        if back == 1000:
            self._price_removed = True
        else:
            self._price_removed = False  # because odds move

    @property
    def progress(self):  # added 09/11/2018
        return self.race_change.progress

    @property
    def progress_seconds_to_finish(self):
        if self.progress and self.speed:
            try:
                return self.progress / self.speed
            except ZeroDivisionError:
                return

    @property
    def speed(self):
        return self.race_change.speed

    @property
    def stride_frequency(self):  # added 09/11/2018
        return self.race_change.stride_frequency

    @property
    def ignore_runner(self):
        if self._price_removed:
            return True
        return False

    @property
    def active(self):
        if self._active and not self._removed:
            return True
        else:
            return False

    @property
    def feed_latency(self):
        if self.race_change:
            return (
                datetime.datetime.utcnow() - self.race_change.feed_time
            ).total_seconds()
        else:
            return 0

    @property
    def selection_id(self):
        return self.race_change.selection_id

    def __str__(self):
        return "StatumRunner {0} {1}".format(self.selection_id, self.active)
